/* ---------------------------------------------------------------------
     Small Music Information Retrieval Toolkit

    Copyright (c) 2008 Rebecca Fiebrink, Ge Wang and Perry R. Cook.  
    All rights reserved.
       http://smirk.cs.princeton.edu/
       http://chuck.cs.princeton.edu
       http://soundlab.cs.princeton.edu/

    Portions of SMIRK are based on the design of the Weka data mining
    toolkit, a GPL package.
       http://www.cs.waikato.ac.nz/ml/weka/
       Ian H. Witten and Eibe Frank (2005) "Data Mining: 
       Practical machine learning tools and techniques", 2nd Edition, 	
       Morgan Kaufmann, San Francisco, 2005.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A. 
--------------------------------------------------------------------- */

/**
	2-class AdaBoost where prediction is used to pan
	
	Instructions:
		Edit feature extraction patch according to task
		Hold 1 to record class 1
		Hold 2 to record class 2
		Press 3 to train
		Hold 4 to classify mic input and pan accordingly
		Press 5 to clear all recorded training instances
*/

adc => dac;

// Patch for pitch:
//adc => FFT fft => blackhole;
//fft =^ FeatureCollector fc;

// -OR-
// Patch for vowel/consonant:
adc => FFT fft =^ Centroid c => blackhole;
c =^ FeatureCollector fc;
fft =^ RMS r =^ fc;
fft =^ RollOff ro =^ fc;

//start with both channels muted
0 => dac.chan(0).gain;
0 => dac.chan(1).gain;

// set fft parameters
1024 => fft.size;
Windowing.hann(512) => fft.window;

//Instantiate the classifier and instances
new AdaBoostM1 @=> AdaBoostM1 ab;
new Instances @=> Instances instances;
50 => int T; // # training rounds

//Instantiate HID to get keyboard input
0 => int deviceNum;
HidIn hi;
HidMsg msg;

// open keyboard
if( !hi.openKeyboard( deviceNum ) ) me.exit();
<<< "keyboard '", hi.name(), "' ready" >>>;

// get keyboard input
0 => int stop;
0 => int going;
while (true) {
    //wait on event
    hi => now;
    //Get keyboard input
    while (hi.recv(msg)) {
        if (msg.isButtonDown()) {
            if (! going) {
                1 => going;
                0 => stop;
                
                if (msg.which == 30) {
                    //record class 1            
                    spork ~ record(1);
                }  else if (msg.which == 31) {
                    //record class 2
                    spork ~ record(-1);
                } else if (msg.which == 32) {
                    //train on all recorded
                    spork ~ train();
                } else if (msg.which == 33) {
                    //run! (test)
					spork ~ test();
                } else if (msg.which == 34) {
					<<< "*******CLEAR!!!******">>>;
					new AdaBoostM1 @=> ab;
					new Instances @=> instances;
				}
            }  
        } else {
            //key up: stop training/testing now
            1 => stop;
            0 => going;
        }
    }
} //repeat key input forever.

// train on class c
function void record(int c) {
    
    //pan the training audio according to specified class
    if (c == 1) {
        1 => dac.chan(0).gain;
        0 => dac.chan(1).gain;
    } else {
        1 => dac.chan(1).gain;
        0 => dac.chan(0).gain;
    }
    
    while (! stop) {        
        
        fc.upchuck() @=> UAnaBlob blob;
        new Instance @=> Instance inst;
        new float[blob.fvals().size()] @=> inst.m_fv;
        
        // copy fvals to instance *by value*
        // otherwise, they'll get overwritten by the next upchuck
        for (0 => int j; j < inst.m_fv.size(); j++) {
            blob.fval(j) => inst.m_fv[j];
        }
        c => inst.m_class;
        
        <<< "Record for class: ", c>>>;
        //ab.train(inst);
		instances.add(inst);        

        // advance time; no rush
        .05::second=>now;
    }
    0 => dac.chan(0).gain;
    0 => dac.chan(1).gain;
} 

function void train() {
	<<< "training on " , instances.num_instances() >>>;
	<<< "instances: ">>>;
	instances => ab.set_instances;
	T => ab.set_num_rounds;
	ab.train();	
	<<< "done training\n">>>;
}

function void test() {
    0 => int count1;
    0 => int count2;
    while (! stop) {
        fc.upchuck() @=> UAnaBlob blob; 
        new Instance @=> Instance inst;
        new float[blob.fvals().size()] @=> inst.m_fv;        
        // copy fvals to instance *by value*
        for (0 => int j; j < inst.m_fv.size(); j++) {
            blob.fval(j) => inst.m_fv[j];
        }
        
        ab.classify(inst) => int c2;
        if (c2 == 1) {
            count1 + 1=> count1;
        } else {
            count2 + 1=> count2;
        }
        <<<"class:",c2>>>;
        
         if (c2 == 1) {
             1 => dac.chan(0).gain;
             0 => dac.chan(1).gain;
        } else {
            1 => dac.chan(1).gain;
            0 => dac.chan(0).gain;
         }
        
        //advance time, do this slower!
        .05::second => now;
        
    }
    <<<"Stopped. final test count: class 1:", count1, "; class 2:", count2>>>;
    0 => dac.chan(0).gain;
    0 => dac.chan(1).gain;
}
