/* ---------------------------------------------------------------------
     Small Music Information Retrieval Toolkit

    Copyright (c) 2008 Rebecca Fiebrink, Ge Wang and Perry R. Cook.  
    All rights reserved.
       http://smirk.cs.princeton.edu/
       http://chuck.cs.princeton.edu
       http://soundlab.cs.princeton.edu/

    Portions of SMIRK are based on the design of the Weka data mining
    toolkit, a GPL package.
       http://www.cs.waikato.ac.nz/ml/weka/
       Ian H. Witten and Eibe Frank (2005) "Data Mining: 
       Practical machine learning tools and techniques", 2nd Edition, 	
       Morgan Kaufmann, San Francisco, 2005.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A. 
--------------------------------------------------------------------- */

/**
 Instances: Holds a collection of instances to be used by 
 a learning algorithm
*/
public class Instances
{
	// An array of instances
    Instance m_instances[0];

	//Keep track of the allowable class values
    0 => int m_max_class;
    0 => int m_min_class;

	//Each instance should have the same number of features
    0 => int m_num_features;

	//Get the number of instances in the set
	fun int num_instances() {
		return m_instances.size();
	}


	//Subsume a new set of instances into this one
	//Adding is done by reference, so take care to copy old set of instances if you want to modify the old set later!
	fun void add(Instances is) {
		//Check that new instances have the same number of features
		//(Assume that new instances is internally consistent in number of features)
		if (is.num_instances() > 0 && m_instances.size() > 0 && (is.m_instances[0].num_feats() != m_num_features)) {
			<<< "Error: mismatch in feature dimensionality: ", is.m_instances[0].num_feats(), m_num_features >>>;
			return;
		}

		for (0 => int i; i < is.num_instances(); i++) {
			add(is.m_instances[i]);
		}
	}

	//Add a new instance to the set
	//Adding is done by reference, so take care to copy old instance if you want to modify it later!
    fun void add(Instance i) {

		//If this is the first instance, initalize this set's fields accordingly
        if (m_instances.size() == 0) {
            m_instances << i;
            i.m_fv.size() => m_num_features;
            i.m_class => m_max_class;
            i.m_class => m_min_class;
            
        } else {
            if (i.m_fv.size() != m_num_features) {
                <<< "Error: mismatch in feature dimensionality: ", i.m_fv.size(), m_num_features>>>;
                return;
            }
            if (i.m_class > m_max_class) {
                i.m_class => m_max_class;
            }
            if (i.m_class < m_min_class) {
                i.m_class => m_min_class;
            }
             
			//Add by reference       
            m_instances << i;
        }
    }
    
	//Clear all instances, creating an empty set
	//This will not delete instances, so they are accessible if referenced elsewhere
    fun void clear() {
        0 => m_instances.size;
        0 => m_max_class => m_min_class;
        0 => m_num_features;
    }
    
	//Print set of instances to the console
    fun void print() {
        for (0=>int j; j< m_instances.size(); j++) {
            //print next
            <<< "Instance ", j>>>;
            m_instances[j].print();
        }
    }
}